@@ -34,7 +34,7 @@
 #include "py/runtime.h"
 #include "py/stream.h"
 #include "extmod/misc.h"
-#include "lib/utils/pyexec.h"
+#include "shared/runtime/pyexec.h"
 
 STATIC byte stdin_ringbuf_array[256];
 ringbuf_t stdin_ringbuf = {stdin_ringbuf_array, sizeof(stdin_ringbuf_array), 0, 0};
@@ -62,6 +62,9 @@ uintptr_t mp_hal_stdio_poll(uintptr_t poll_flags) {
     if ((poll_flags & MP_STREAM_POLL_RD) && stdin_ringbuf.iget != stdin_ringbuf.iput) {
         ret |= MP_STREAM_POLL_RD;
     }
+    if (poll_flags & MP_STREAM_POLL_WR) {
+        ret |= mp_uos_dupterm_poll(poll_flags);
+    }
     return ret;
 }
 
@@ -91,33 +94,10 @@ void mp_hal_debug_str(const char *str) {
 }
 #endif
 
-void mp_hal_stdout_tx_str(const char *str) {
-    mp_uos_dupterm_tx_strn(str, strlen(str));
-}
-
 void mp_hal_stdout_tx_strn(const char *str, uint32_t len) {
     mp_uos_dupterm_tx_strn(str, len);
 }
 
-void mp_hal_stdout_tx_strn_cooked(const char *str, uint32_t len) {
-    const char *last = str;
-    while (len--) {
-        if (*str == '\n') {
-            if (str > last) {
-                mp_uos_dupterm_tx_strn(last, str - last);
-            }
-            mp_uos_dupterm_tx_strn("\r\n", 2);
-            ++str;
-            last = str;
-        } else {
-            ++str;
-        }
-    }
-    if (str > last) {
-        mp_uos_dupterm_tx_strn(last, str - last);
-    }
-}
-
 void mp_hal_debug_tx_strn_cooked(void *env, const char *str, uint32_t len) {
     (void)env;
     while (len--) {
@@ -139,7 +119,7 @@ void MP_FASTCODE(mp_hal_delay_ms)(uint32_t delay) {
 }
 
 uint64_t mp_hal_time_ns(void) {
-    return pyb_rtc_get_us_since_2000() * 1000ULL;
+    return pyb_rtc_get_us_since_epoch() * 1000ULL;
 }
 
 void ets_event_poll(void) {
