@@ -42,7 +42,7 @@ static ringbuf_t uart_ringbuf = {uart_ringbuf_array, sizeof(uart_ringbuf_array),
 static void uart0_rx_intr_handler(void *para);
 
 void soft_reset(void);
-void mp_keyboard_interrupt(void);
+void mp_sched_keyboard_interrupt(void);
 
 /******************************************************************************
  * FunctionName : uart_config
@@ -111,6 +111,15 @@ void uart_tx_one_char(uint8 uart, uint8 TxChar) {
     WRITE_PERI_REG(UART_FIFO(uart), TxChar);
 }
 
+int uart_txdone(uint8 uart) {
+    uint32 fifo_cnt = READ_PERI_REG(UART_STATUS(uart)) & (UART_TXFIFO_CNT << UART_TXFIFO_CNT_S);
+    if ((fifo_cnt >> UART_TXFIFO_CNT_S & UART_TXFIFO_CNT) == 0) {
+        return true;
+    } else {
+        return false;
+    }
+}
+
 void uart_flush(uint8 uart) {
     while (true) {
         uint32 fifo_cnt = READ_PERI_REG(UART_STATUS(uart)) & (UART_TXFIFO_CNT << UART_TXFIFO_CNT_S);
@@ -179,7 +188,7 @@ static void uart0_rx_intr_handler(void *para) {
             // directly on stdin_ringbuf, rather than going via uart_ringbuf
             if (uart_attached_to_dupterm) {
                 if (RcvChar == mp_interrupt_char) {
-                    mp_keyboard_interrupt();
+                    mp_sched_keyboard_interrupt();
                 } else {
                     ringbuf_put(&stdin_ringbuf, RcvChar);
                 }
@@ -285,7 +294,7 @@ void ICACHE_FLASH_ATTR uart0_set_rxbuf(uint8 *buf, int len) {
 // Task-based UART interface
 
 #include "py/obj.h"
-#include "lib/utils/pyexec.h"
+#include "shared/runtime/pyexec.h"
 
 #if MICROPY_REPL_EVENT_DRIVEN
 void ICACHE_FLASH_ATTR uart_task_handler(os_event_t *evt) {
@@ -303,7 +312,7 @@ void ICACHE_FLASH_ATTR uart_task_handler(os_event_t *evt) {
     int c, ret = 0;
     while ((c = ringbuf_get(&stdin_ringbuf)) >= 0) {
         if (c == mp_interrupt_char) {
-            mp_keyboard_interrupt();
+            mp_sched_keyboard_interrupt();
         }
         ret = pyexec_event_repl_process_char(c);
         if (ret & PYEXEC_FORCED_EXIT) {
